@@ -3,16 +3,26 @@ import traceback
 import uuid
 import time
 
+from selenium.common.exceptions import NoSuchElementException
+from datetime import datetime
 from jaaql.constants import *
+from urllib.parse import quote
+from jaaql.exceptions.http_status_exception import HttpStatusException
 from jaaql.utilities.utils import load_config, await_jaaql_installation, get_jaaql_connection
-from jaaql.utilities.utils_no_project_imports import check_allowable_file_path
+from jaaql.utilities.utils_no_project_imports import check_allowable_file_path, time_delta_ms
 from email.utils import formatdate, make_msgid
 from smtplib import SMTP, SMTPException, SMTPAuthenticationError
+import base64
 from base64 import urlsafe_b64encode as b64e
 from typing import Union, List
 from email.mime.multipart import MIMEMultipart
 from email.mime.text import MIMEText
+from email.mime.application import MIMEApplication
 from queue import Queue, Empty
+from selenium import webdriver
+from selenium.webdriver.common.by import By
+from selenium.webdriver.chrome.options import Options
+from selenium.webdriver.chrome.service import Service
 import re
 import hashlib
 import json
@@ -24,14 +34,14 @@ from jaaql.mvc.generated_queries import *
 from jaaql.utilities.vault import Vault, DIR__vault
 from jaaql.constants import VAULT_KEY__db_crypt_key
 
-ELE__jaaql_filename = "jaaql_filename"
+ATTR__finished_request = "data-jaaql-finished-request"
+ATTR__finished_request_success = "data-jaaql-finished-request-success"
 
 ERR__email_not_found = "Email account not found with name '%s'"
 ERR__invalid_call_to_internal_email_service = "Invalid call to internal email service"
-ERR__attachment_timeout = "Could not send email! Attachment rendering timed out!"
+ERR__attachment_timeout = "Could not send email! Attachment took too long to render!"
+ERR__attachment_timeout_render = "Could not send email! Attachment timed out whilst rendering!"
 ERR__attachment_error = "Could not send email! Attachment rendering received error: '%s'"
-ERR__attachment_filename = "Attachment did not render in time or could not find the file name for the attachment. Please check template. Expected " \
-    "presence of element with id " + ELE__jaaql_filename + " to be present after page has finished loading"
 
 PROTOCOL__imap = "imap"
 PROTOCOL__smtp = "smtp"
@@ -46,16 +56,109 @@ EMAIL__message_id = "Message-ID"
 SPLIT__address = ", "
 
 TIMEOUT__attachment = 60000
-TIMEOUT__filename = 15000
+TIMEOUT__attachment_render = 15000
 
 REGEX__html_tags = r'<(\/[a-zA-Z]+|(!?[a-zA-Z]+((\s)*((\"?[^(\">)]*\"?)|[a-zA-Z]*\s*=\s*\"[^\"]*\"))*))>'
 
 HTTP_ARG__oauth_token = "oauth_token"
 
+KEY__attachment_name = "name"
+KEY__attachment_application = "application"
+KEY__artifact_base = "artifact_base"
+
+
+class DrivenChrome:
+    def __init__(self, db_interface: DBInterface, db_key: bytes, is_deployed: bool):
+        self.attachments = Queue()
+        self.completed_attachments = set()
+        self.errored_attachments = {}
+        self.db_interface = db_interface
+        self.chrome_lock = threading.Lock()
+        self.driver = None
+        self.is_deployed = is_deployed
+        self.db_key = db_key
+        self.template_dir_path = os.path.join(DIR__www, DIR__render_template)
+
+        self.a4_params = {'landscape': False, 'paperWidth': 8.27, 'paperHeight': 11.69, 'printBackground': True}
+
+        threading.Thread(target=self.start_chrome, daemon=True).start()
+
+    def parameters_to_get_str(self, access_token: str, parameters: dict):
+        if parameters is None:
+            parameters = {}
+        parameters[KEY__oauth_token] = access_token
+
+        return "?" + "&".join([key + "=" + quote(itm) for key, itm in parameters.items()])
+
+    def chrome_page_to_pdf(self, url: str, access_token: str, parameters: dict):
+        with self.chrome_lock:
+            self.driver.get(url + self.parameters_to_get_str(access_token, parameters))
+            start_time = datetime.now()
+            while True:
+                passed = False
+                filename = ""
+                try:
+                    body = self.driver.find_element(By.TAG_NAME, "body")  # Sometimes the body is not yet present, in which case an exception will fire
+                    finished = body.get_attribute(ATTR__finished_request)
+                    finished_success = body.get_attribute(ATTR__finished_request_success)
+                    if finished_success is not None:
+                        passed = True
+                        if finished_success != ATTR__finished_request_success:
+                            filename = finished_success
+                    if finished is not None and not passed:
+                        raise HttpStatusException("Application level exception: '%s'" % base64.b64decode(finished).decode("UTF-8"))
+                except NoSuchElementException:
+                    pass
+
+                if passed:
+                    if len(filename) == 0:
+                        filename = url.split("/")[-1].split(".html")[0]
+                    if not filename.endswith(".pdf"):
+                        filename += ".pdf"
+                    break
+                else:
+                    if time_delta_ms(start_time, datetime.now()) > TIMEOUT__attachment_render:
+                        raise HttpStatusException(ERR__attachment_timeout_render)
+
+            return filename, base64.b64decode(self.driver.execute_cdp_cmd("Page.printToPDF", self.a4_params)["data"])
+
+    def start_chrome(self):
+        options = Options()
+        options.add_argument("--window-size=1920,1080")
+        options.headless = True
+        service_log_path = None
+
+        if self.is_deployed:
+            options.add_argument('--no-sandbox')
+            service_log_path = "log/chromedriver.log"
+
+        with webdriver.Chrome(options=options, service=Service(service_log_path=service_log_path)) as driver:
+            self.driver = driver
+            while True:
+                current_attachment: 'EmailAttachment' = self.attachments.get()
+                try:
+                    template = document_template__select(self.db_interface, current_attachment.application, current_attachment.name)
+                    content_path = template[KG__document_template__content_path]
+                    if not content_path.endswith(".html"):
+                        content_path += ".html"
+                    content_path = current_attachment.artifact_base + "/" + content_path
+                    filename, content = self.chrome_page_to_pdf(content_path, current_attachment.access_token, current_attachment.parameters)
+                    current_attachment.content = content
+                    current_attachment.filename = filename
+                except HttpStatusException as ex:
+                    self.errored_attachments[current_attachment.id] = str(ex)
+                    self.completed_attachments.add(current_attachment.id)
+                except Exception as ex:
+                    traceback.print_exc()
+                    self.errored_attachments[current_attachment.id] = str(ex)
+                    self.completed_attachments.add(current_attachment.id)
+                self.completed_attachments.add(current_attachment.id)
+
 
 class EmailAttachment:
-    def __init__(self, name: str, parameters: dict, template: str, artifact_base: str = None):
+    def __init__(self, name: str, application: str, parameters: dict, template: str, artifact_base: str = None):
         self.name = name
+        self.application = application
         self.artifact_base = artifact_base
         self.parameters = parameters
         self.id = str(uuid.uuid4())
@@ -69,12 +172,12 @@ class EmailAttachment:
         if not artifact_base_url.startswith("https://") and not artifact_base_url.startswith("http://"):
             if is_container:
                 if check_allowable_file_path(artifact_base_url):
-                    raise Exception("Illegal artifact base url")
+                    raise Exception("Illegal artifact base url: '" + artifact_base_url + "'")
 
             if artifact_base_url.startswith("file:///"):
                 return artifact_base_url
             else:
-                return "file://" + os.environ[ENVIRON__install_path] + "/www/" + artifact_base_url
+                return "file://" + os.environ.get(ENVIRON__install_path, os.getcwd().replace("\\", "/")) + "/www" + artifact_base_url
         else:
             return artifact_base_url
 
@@ -82,15 +185,17 @@ class EmailAttachment:
         self.artifact_base = EmailAttachment.static_format_attached_url(artifact_base_url, is_container)
 
     def repr_json(self):
-        return dict(name=self.name, parameters=self.parameters, template=self.template, artifact_base=self.artifact_base)
+        return dict(name=self.name, application=self.application, parameters=self.parameters,
+                    template=self.template, artifact_base=self.artifact_base)
 
     @staticmethod
     def deserialize(attachment: dict, template: str = None, artiface_base_url: str = None):
         if template is not None:
-            return EmailAttachment(attachment[KEY__attachment_name], attachment[KEY__parameters], template, artiface_base_url)
+            return EmailAttachment(attachment[KEY__attachment_name], attachment[KEY__attachment_application], attachment[KEY__parameters],
+                                   template, artiface_base_url)
         else:
-            return EmailAttachment(attachment[KEY__attachment_name], attachment[KEY__parameters], attachment[KEY__template],
-                                   attachment[KEY__artifact_base])
+            return EmailAttachment(attachment[KEY__attachment_name], attachment[KEY__attachment_application], attachment[KEY__parameters],
+                                   attachment[KEY__template], attachment[KEY__artifact_base])
 
     @staticmethod
     def deserialize_list(attachments: list, template: str, artiface_base_url: str):
@@ -105,6 +210,33 @@ class EmailAttachment:
     def encode_content(self):
         return b64e(self.content).decode(ENCODING__ascii)
 
+    def build_attachment(self, access_token: str, driven_chrome: DrivenChrome, is_gunicorn: bool) -> MIMEApplication:
+        self.access_token = access_token
+        driven_chrome.attachments.put(self)
+
+        start_time = datetime.now()
+        while time_delta_ms(start_time, datetime.now()) < TIMEOUT__attachment:
+            if self.id in driven_chrome.completed_attachments:
+                break
+            time.sleep(1)
+
+        if self.id not in driven_chrome.completed_attachments:
+            raise Exception(ERR__attachment_timeout)
+        driven_chrome.completed_attachments.remove(self.id)
+
+        if self.id in driven_chrome.errored_attachments:
+            raise Exception(ERR__attachment_error % driven_chrome.errored_attachments.pop(self.id))
+
+        attachment = MIMEApplication(self.content, _subtype=self.filename.split(".")[-1])
+        attachment.add_header('Content-Disposition', 'attachment', filename=self.filename)
+
+        if not is_gunicorn:
+            os.makedirs("rendered_templates", exist_ok=True)
+            with open("rendered_templates/" + self.filename, "wb") as f:
+                f.write(self.content)
+
+        return attachment
+
 
 TYPE__email_attachments = Union[EmailAttachment, List[EmailAttachment]]
 
@@ -202,6 +334,8 @@ class EmailManagerService:
         self.threadsafe_queue_initialiser = ThreadsafeQueueInitialiser()
         threading.Thread(target=self.dispatcher_manager_thread, daemon=True).start()
 
+        self.driven_chrome = DrivenChrome(connection, self.db_crypt_key, self.is_gunicorn)
+
     def construct_message(self, dispatcher_info: dict, email: Email) -> (str, MIMEMultipart):
         send_body = email.body
 
@@ -227,14 +361,14 @@ class EmailManagerService:
         if email.attachments is not None:
             email_attachments = email.attachments
             for attachment in email_attachments:
-                pass  # TODO set up attachments
+                message_final.attach(attachment.build_attachment(email.attachment_access_token, self.driven_chrome, self.is_gunicorn))
 
         return send_body, message_final
 
     def is_connected(self, conn: SMTP):
         try:
             status = conn.noop()[0]
-        except Exception:
+        except:
             status = -1
         return True if status == 250 else False
 
@@ -279,11 +413,13 @@ class EmailManagerService:
         for dispatcher_key, dispatcher_tuple in potentially_require_threads.items():
             threading.Thread(target=self.dispatcher_thread, args=[dispatcher_key, dispatcher_tuple[0]]).start()
 
-    def get_dispatcher_queue_from_key(self, dispatcher_key: str):
-        return self.dispatchers.get(dispatcher_key, (None, self.threadsafe_queue_initialiser.fetch_queue(dispatcher_key)))[1]
+    def get_dispatcher_queue_from_key(self, dispatcher_key: str) -> Queue:
+        _, ret = self.dispatchers.get(dispatcher_key, (None, self.threadsafe_queue_initialiser.fetch_queue(dispatcher_key)))
+        return ret
 
-    def get_dispatcher_info_from_key(self, dispatcher_key: str):
-        return self.dispatchers.get(dispatcher_key, (None, None))[0]
+    def get_dispatcher_info_from_key(self, dispatcher_key: str) -> Union[dict, None]:
+        info, _ = self.dispatchers.get(dispatcher_key, (None, None))
+        return info
 
     def dispatcher_manager_thread(self):
         printed = False
